import { TypeKeys } from "web-utility";
export type HTTPMethod = 'GET' | 'POST' | 'PUT' | 'PATCH' | 'DELETE';
export type HTTPAuthorizationSchema = 'Basic' | 'Bearer' | 'Digest' | 'HOBA' | 'Mutual' | 'Negotiate' | 'OAuth' | 'SCRAM-SHA-1' | 'SCRAM-SHA-256' | 'vapid';
export type IDType = number | string;
export type NewData<C, P> = Partial<Omit<C, TypeKeys<C, P> | TypeKeys<C, P[]>> & {
    [K in TypeKeys<C, P>]: IDType;
} & {
    [K in TypeKeys<C, P[]>]: IDType[];
}>;
export function Body(): ParameterDecorator;
export function checkInput(): (target: any, key: string, meta: PropertyDescriptor) => void;
export interface PageData<T> {
    keyword?: string;
    pageIndex: number;
    pageSize: number;
    totalCount: number;
    list: T[];
}
export type PageFilter<C, P> = Omit<PageData<C>, 'list'> & NewData<C, P>;
export abstract class BaseStore<C, P, F extends PageFilter<C, P> = PageFilter<C, P>> implements PageData<C> {
    static setAuthorization?: (value: string, type?: HTTPAuthorizationSchema) => any;
    abstract request<T = C>(method: HTTPMethod, path: string, body?: any, header?: Record<string, string>): Promise<T>;
    abstract requestPage(path: string, filter?: F): Promise<PageData<C>>;
    abstract path: string;
    loading: boolean;
    updating: boolean;
    current: C;
    keyword?: string;
    pageIndex: number;
    pageSize: number;
    totalCount: number;
    list: C[];
    get totalPage(): number;
    get noMore(): boolean;
    clearCurrent(): void;
    clearList(): void;
    updateOne(data: NewData<C, P>, id?: IDType): Promise<C>;
    getOne(id: IDType): Promise<C>;
    deleteOne(id: IDType): Promise<void>;
    getList(filter?: F): Promise<C[]>;
}
export function toggle<T extends BaseStore<any, any> = BaseStore<any, any>>(statusKey: TypeKeys<T, boolean>): (target: any, key: string, meta: PropertyDescriptor) => void;

//# sourceMappingURL=index.d.ts.map
